"use strict";
/// <reference lib="dom" />
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchAdapter = void 0;
const axios_1 = require("axios");
const fetch_utils_1 = require("./fetch.utils");
/**
 * fetch adapter for axios
 * Contrary to several other adapters, this one does not rely on axios internal helpers that needs contant refactoring and maintenance
 * Instead, it implements some basic helpers itself to make it work with axios. This will make this adapter easier to maintain and extend
 * @param conf AxiosFetchRequestConfig
 * @returns
 */
const fetchAdapter = (conf) => __awaiter(void 0, void 0, void 0, function* () {
    const config = conf;
    const request = createFetchRequest(config);
    const response = yield fetchRequest(request, config);
    const axiosResponse = yield getAxiosResponse(request, response, config);
    if (config.validateStatus && !config.validateStatus(axiosResponse.status)) {
        throw new axios_1.AxiosError(axiosResponse.statusText, `${axiosResponse.status}`, config, request, axiosResponse);
    }
    return axiosResponse;
});
exports.fetchAdapter = fetchAdapter;
// istanbul ignore next
function createFetchRequest(config) {
    var _a, _b, _c, _d;
    if ((0, fetch_utils_1.isBrowser)() &&
        ((0, fetch_utils_1.isFormData)(config.data) || (0, fetch_utils_1.isBlob)(config.data) || (0, fetch_utils_1.isFile)(config.data)) &&
        ((_a = config.headers) === null || _a === void 0 ? void 0 : _a["Content-Type"])) {
        delete config.headers["Content-Type"];
    }
    const headers = new Headers(config.headers);
    if (config.auth) {
        const username = config.auth.username || "";
        const password = config.auth.password
            ? decodeURI(encodeURIComponent(config.auth.password))
            : "";
        headers.set("Authorization", `Basic ${btoa(username + ":" + password)}`);
    }
    if (config.xsrfHeaderName && config.xsrfCookieName) {
        const xsrfValue = config.withCredentials || config.credentials !== "omit"
            ? (0, fetch_utils_1.findCookieByName)(config.xsrfCookieName)
            : undefined;
        if (xsrfValue) {
            headers.set(config.xsrfHeaderName, xsrfValue);
        }
    }
    const fetchOptions = {
        method: ((_b = config.method) === null || _b === void 0 ? void 0 : _b.toUpperCase()) || "GET",
        headers,
        body: config.data,
        signal: config.signal,
        mode: config.mode,
        cache: config.cache,
        integrity: config.integrity,
        keepalive: config.keepalive,
        redirect: ((_c = config.redirect) !== null && _c !== void 0 ? _c : config.maxRedirects === 0) ? "manual" : "follow",
        referrer: config.referrer,
        referrerPolicy: config.referrerPolicy,
        credentials: ((_d = config.credentials) !== null && _d !== void 0 ? _d : config.withCredentials === undefined)
            ? "same-origin"
            : config.withCredentials
                ? "include"
                : "omit",
    };
    const url = (0, fetch_utils_1.buildURL)(config);
    return new Request(url, fetchOptions);
}
/**
 * Simple wrapper for fetch request that transforms fetch errors into axios errors
 * @param request - fetch Request
 * @param config - AxiosFetchRequestConfig
 * @returns fetch Response
 */
function fetchRequest(request, config) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            return yield fetch(request);
        }
        catch (error) {
            // istanbul ignore next
            if (error instanceof Error) {
                throw new axios_1.AxiosError(error.message, axios_1.AxiosError.ERR_NETWORK, config, request);
            }
            else {
                throw new axios_1.AxiosError("Network Error", axios_1.AxiosError.ERR_NETWORK, config, request);
            }
        }
    });
}
/**
 * Simple wrapper for fetch response that transforms fetch headers and response format into axios one
 * @param request - A fetch Request
 * @param response - A fetch Response
 * @param config - AxiosFetchRequestConfig
 * @returns transformed axios response
 */
function getAxiosResponse(request, response, config) {
    return __awaiter(this, void 0, void 0, function* () {
        // @ts-ignore
        const headers = {};
        response.headers.forEach((value, name) => {
            headers[name.toLowerCase()] = value;
        });
        const result = {
            request,
            headers,
            config,
            status: response.status,
            statusText: response.statusText,
            data: yield getResponseData(response, config),
        };
        return result;
    });
}
// istanbul ignore next
function getResponseData(response, config) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        switch (config.responseType) {
            case "arraybuffer":
                return response.arrayBuffer();
            case "blob":
                return response.blob();
            case "document":
                throw new Error("document response type not supported for fetch");
            case "stream":
                return response.body;
            case "text":
                return response.text();
            case "json":
                return response.json();
            default: {
                if ((_a = response.headers.get("content-type")) === null || _a === void 0 ? void 0 : _a.includes("application/json")) {
                    return response.json();
                }
                return response.text();
            }
        }
    });
}
